﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Web.UI;
using System.Web.UI.WebControls;
using Telerik.Sitefinity.Data.Metadata;
using Telerik.Sitefinity.Metadata.Model;
using Telerik.Sitefinity.Model;
using Telerik.Sitefinity.Modules.Forms.Web.UI.Fields;
using Telerik.Sitefinity.Security;
using Telerik.Sitefinity.Utilities.TypeConverters;
using Telerik.Sitefinity.Web.UI;
using Telerik.Sitefinity.Web.UI.ControlDesign;
using Telerik.Sitefinity.Web.UI.Fields;
using Telerik.Sitefinity.Web.UI.Fields.Enums;

namespace SitefinityWebApp.FormFields
{
    [DatabaseMapping(UserFriendlyDataType.ShortText)]
    [PropertyEditorTitle("FormHiddenField Properties"), Telerik.Sitefinity.Web.UI.ControlDesign.ControlDesigner(typeof(SitefinityWebApp.FormFields.Designers.FormHiddenFieldDesigner))]
    public class FormHiddenField : FieldControl, IFormFieldControl
    {
        #region Constructor
        /// <summary>
        /// Initializes a new instance of the FormHiddenField class.
        /// </summary>
        public FormHiddenField()
        {
            this.Title = "Hidden Field";
        }
        #endregion

        #region Public properties (will show up in dialog)
        /// <summary>
        /// Hidden value
        /// </summary>
        public string HiddenValue { get; set; }
        #endregion

        #region IFormFieldControl members
        /// <summary>
        /// Gets or sets MetaField property to persist data from control to the DB when form is submitted
        /// </summary>
        [TypeConverter(typeof(ExpandableObjectConverter))]
        public IMetaField MetaField
        {
            get
            {
                if (this.metaField == null)
                {
                    this.metaField = this.LoadDefaultMetaField();
                }

                return this.metaField;
            }
            set
            {
                this.metaField = value;
            }
        }
        #endregion

        /// <summary>
        /// Gets or sets the default value.
        /// </summary>
        /// <value>The default value.</value>
        public override object DefaultValue
        {
            get
            {
                if (string.IsNullOrEmpty(base.DefaultValue as string))
                {
                    return this.Value;
                }
                return base.DefaultValue;
            }
            set
            {
                base.DefaultValue = value;
            }
        }

        #region Value method
        /// <summary>
        /// Get and set the value of the field. 
        /// </summary>
        [TypeConverter(typeof(ObjectStringConverter))]
        public override object Value
        {
            get
            {
                return this.HiddenField1.Value;
            }

            set
            {
                this.HiddenField1.Value = value as string;
            }
        }
        #endregion

        #region Template
        /// <summary>
        /// Obsolete. Use LayoutTemplatePath instead.
        /// </summary>
        protected override string LayoutTemplateName
        {
            get
            {
                return string.Empty;
            }
        }

        /// <summary>
        /// Gets the layout template's relative or virtual path.
        /// </summary>
        public override string LayoutTemplatePath
        {
            get
            {
                if (string.IsNullOrEmpty(base.LayoutTemplatePath))
                    return FormHiddenField.layoutTemplatePath;
                return base.LayoutTemplatePath;
            }
            set
            {
                base.LayoutTemplatePath = value;
            }
        }
        #endregion
        
        #region Controls on control template
        /// <summary>
        /// Gets reference to the DescriptionLabel
        /// </summary>
        protected internal virtual Label Label1
        {
            get
            {
                return Container.GetControl<Label>("Label1", true);
            }
        }
        /// <summary>
        /// Reference to the Label1
        /// </summary>
        protected WebControl LabelControl
        {
            get
            {
                return this.Label1;
            }
        }
        #endregion
        
        #region HiddenField on control
        /// <summary>
        /// Gets reference to the HiddenField1 control
        /// </summary>
        protected virtual HiddenField HiddenField1
        {
            get
            {
                return this.Container.GetControl<HiddenField>("HiddenField1", true);
            }
        }
        #endregion

        #region InitializeControls method
        /// <summary>
        /// Initializes the controls.
        /// </summary>
        /// <remarks>
        /// Initialize your controls in this method. Do not override CreateChildControls method.
        /// </remarks>
        protected override void InitializeControls(GenericContainer container)
        {
            this.Label1.Visible = false;
            this.HiddenField1.Value = this.HiddenValue;

            if (this.IsDesignMode())
            {
                this.Label1.Text = String.Format("<strong>{0}</strong><br />{1}", this.Title, this.HiddenField1.Value);
                this.Label1.Visible = true;
            }
        }
        #endregion

        #region Script methods
        /// <summary>
        /// Get list of all scripts used by control
        /// </summary>
        /// <returns>List of all scripts used by control</returns>
        public override IEnumerable<ScriptDescriptor> GetScriptDescriptors()
        {
            var descriptor = new ScriptControlDescriptor(typeof(FormHiddenField).FullName, this.ClientID);
            descriptor.AddElementProperty("hiddenField", this.HiddenField1.ClientID);
            descriptor.AddProperty("dataFieldName", this.MetaField.FieldName); //the field name of the corresponding widget
            descriptor.AddProperty("fieldName", this.FieldName);
            return new[] { descriptor };
        }

        /// <summary>
        /// Get reference to all scripts
        /// </summary>
        /// <returns>Reference to all scripts</returns>
        public override IEnumerable<System.Web.UI.ScriptReference> GetScriptReferences()
        {
            var scripts = new List<ScriptReference>(base.GetScriptReferences())
            {
                new ScriptReference(FormHiddenField.scriptReference),
            };
            return scripts;
        }
        #endregion

        #region Private fields and constants
        private IMetaField metaField = null;
        public static readonly string layoutTemplatePath = "~/FormFields/FormHiddenField/FormHiddenField.ascx";
        public const string scriptReference = "~/FormFields/FormHiddenField/FormHiddenField.js";
        #endregion
    }
}